#!/bin/tcsh -f

# ---------------------------------------------------------------------------
#
# try -help for details
#
# ---------------------------------------------------------------------------
# author                                    mihir arjunwadkar (mihir@cmu.edu)
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

# defaults
#
 set myname = ${0:t}

# temporary files prefix
#
 mkdir -p /tmp/${user}
 set tmpdir = /tmp/${user}/
 set tmp    = ${tmpdir}`echo ${myname} | sed 's/\.csh$//'`.`date '+%m%d%T' | sed 's/://g'`

# verbosity
#
 set verbmax   = 1
 set verbosity = ${verbmax}

# brain output prefix, submodels to process
#
 set prefix = './'
 set submodels = ''

# probabilities file/chunk names
#
 set mpchunk   = model.probabilities

 set outchunk  = ${mpchunk}
 set outprefix = ''
 set outsuffix = ".${outchunk}"

# command line args
#
 if ( ${#argv} == 0 ) set argv = -help

 while ( ${#argv} > 0 )
  switch ( ${argv[1]} )
   case "-h":
   case "-help":
    echo ${0}
    echo ' Extract model probabilities for specified'
    echo ' submodel from vmp output.'
    echo ''
    echo 'Command line'
    echo ' [-h|-help]'
    echo ' [-q|-quiet] [-v|-verbosity level] [-V|-verbose]'
    echo ''
    echo ' [-p|-prefix input_prefix]'
    echo ' [-op|-outprefix output_prefix]'
    echo ' [-os|-outsuffix output_suffix]'
    echo ' [-oc|-outchunk output_chunk_name]'
    echo ' [submodel1 submodel2 ...]'
    echo ''
    echo 'Termination status codes'
    echo ' 0: normal termination'
    echo ' 1: abnormal termination, except'
    echo ' 2-: failed utility/script'
    echo ''
    echo 'Required utilities/scripts'
    echo ' errmsg.csh'
    echo ' mri.rename.chunk.csh'
    echo ' mri_printfield mri_subset'
    echo ''
    echo 'Notes'
    echo " 1 Produces one file per submodel; submodel, file and chunk names are"
    echo "   printed on stdout,  along with  global and  within-block parameter"
    echo "   offsets and block name."
    echo " 2 Output file names are of the form"
    echo "    (output_prefix)(submodel)(output_suffix).mri"
#   echo ''
#   echo 'Defaults'
#   echo " prefix: '${prefix}'"
#   echo ''
#   echo " verbosity: ${verbosity} (maximum ${verbmax})"
#   echo " temporary files prefix: ${tmp}"
    exit 0;

   case "-q":
   case "-quiet":
    set verbosity = 0; shift; breaksw;

   case "-v":
   case "-verbosity":
    set verbosity = ${argv[2]}; shift; shift; breaksw;

   case "-V":
   case "-verbose":
    set verbosity = ${verbmax}; shift; breaksw;

   case "-p":
   case "-prefix":
    set prefix = `echo ${argv[2]} | sed 's/\.mri$//'`;
    shift; shift; breaksw;

   case "-oc":
   case "-outchunk":
    set outchunk = ${argv[2]}; shift; shift; breaksw;

   case "-op":
   case "-outprefix":
    set outprefix = ${argv[2]}; shift; shift; breaksw;

   case "-os":
   case "-outsuffix":
    set outsuffix = ${argv[2]}; shift; shift; breaksw;

   default:
    set submodels = ( ${submodels} ${argv[1]} ); shift; breaksw;
  endsw
 end

# submodels
#
 set fitsubmodels = ( `mri_printfield -input ${prefix}${mpchunk} -field submodel.list` )

 errmsg.csh ${verbosity} ${myname} 2 "failed command" ${status} "mri_printfield -input ${prefix}${mpchunk} -field submodel.list"
 if ( ${status} ) exit ${status}

 set sm        = ( ${submodels} )
 set submodels = ''
 foreach s ( ${sm} )
  if ( `echo '' | awk '{print index("'"${fitsubmodels}"'","'${s}'")}'` != 0 ) set submodels = ( ${submodels} ${s} )
 end

 if ( "${submodels}" == "" ) set submodels = ( ${fitsubmodels} )

# loop over submodels
#
 foreach submodel ( ${submodels} )
  if ( ${submodel} == average ) goto label_skipaverage

  # output file name
  #
  set outfile = ${outprefix}${submodel}${outsuffix}

  # offset for ${submodel} in chunk ${mpchunk}
  #
  set offset = `mri_printfield -input ${prefix}${mpchunk}.mri -field submodel.index.${submodel}`

  errmsg.csh ${verbosity} ${myname} 3 "failed command" ${status} "mri_printfield -input ${prefix}${mpchunk}.mri -field submodel.index.${submodel}"
  if ( ${status} ) exit ${status}

  # rename model probabilities chunk to images
  # (mri_subset may not be able to handle chunknames that involve a '.')
  #
  mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${mpchunk}.mri -chunk ${mpchunk} -outfile ${tmp} -to images

  errmsg.csh ${verbosity} ${myname} 4 "failed command" ${status} "mri.rename.chunk.csh -verbosity ${verbosity} -infile ${prefix}${mpchunk}.mri -chunk ${mpchunk} -outfile ${tmp} -to images"
  if ( ${status} ) exit ${status}

  # extract model probabilities for submodel
  #
  mri_subset -d s -s ${offset} -l 1 ${tmp} ${tmp}.mp >& ${tmp}.err

  errmsg.csh ${verbosity} ${myname} 5 "failed command" ${status} "mri_subset -d s -s ${offset} -l 1 ${tmp} ${tmp}.mp >& ${tmp}.err"
  if ( ${status} ) exit ${status}

  # rename "images" back to ${outchunk} if necessary
  #
  mv ${tmp}.mp.dat ${outfile}.dat

  if ( "${outchunk}" == "images" ) then
   grep -v -e 'history\.' ${tmp}.mp.mri > ${outfile}.mri
  else
   grep -v -e 'history\.' ${tmp}.mp.mri | sed 's/^images/'${outchunk}'/g' > ${outfile}.mri
  endif

  echo "submodel.list = ${submodel}"    >> ${outfile}.mri
  echo "submodel.index.${submodel} = 0" >> ${outfile}.mri
  echo "submodel.description.${submodel} = `mri_printfield -input ${prefix}${mpchunk}.mri -field submodel.description.${submodel}`" >> ${outfile}.mri
  echo "imagedata.description = `mri_printfield -input ${prefix}${mpchunk}.mri -field imagedata.description`" >> ${outfile}.mri
  echo "thisrun.description = `mri_printfield -input ${prefix}${mpchunk}.mri -field thisrun.description`" >> ${outfile}.mri

  echo "submodel: ${submodel} file: ${outfile} chunk: ${outchunk}"

  label_skipaverage:
 end

# cleanup
#
 rm -f ${tmp}* >& /dev/null; exit 0
